# End users don't need to worry about this file.
#
# Version:: $Id: base.rb,v 1.9 2002/01/15 08:52:42 cepheus Exp $

require "log4r/config"

module Log4r
  ALL = 0
  LNAMES = ['ALL']

  #--
  # Defines the log levels of the Log4r module at runtime. It is given
  # either the default level spec (when root logger is created) or the
  # user-specified level spec (when Logger.custom_levels is called).
  #
  # The last constant defined by this method is OFF. Other level-sensitive 
  # parts of the code check to see if OFF is defined before deciding what 
  # to do. The typical action would be to force the creation of RootLogger 
  # so that the custom levels get loaded and business can proceed as usual.
  #
  # For purposes of formatting, a constant named MaxLevelLength is defined
  # in this method. It stores the max level name string size.

  def Log4r.define_levels(*levels)
    return if const_defined? :OFF
    for i in 0...levels.size
      name = levels[i].to_s
      module_eval "#{name} = #{i} + 1; LNAMES.push '#{name}'"
    end
    module_eval %{
      LNAMES.push 'OFF'
      LEVELS = LNAMES.size
      OFF = LEVELS - 1
      MaxLevelLength = Log4rTools.max_level_str_size
    }
  end

  # Commonly used functions go here.
  class Log4rTools # :nodoc:
    class << self
      def validate_level(level, depth=0)
        unless valid_level?(level)
          raise ArgumentError, "Log level must be in 0..#{LEVELS}",
                caller[1..-(depth + 1)]
        end
      end
      def valid_level?(lev)
        not lev.nil? and lev.kind_of?(Numeric) and lev >= ALL and lev <= OFF
      end
      def max_level_str_size
        size = 0
        LNAMES.each {|i| size = i.length if i.length > size}
        size
      end
    end
  end
end
