# Version:: $Id: loggerfactory.rb,v 1.8 2002/01/15 14:18:18 cepheus Exp $

require "log4r/base"
require "log4r/repository"

module Log4r
class Logger

  class LoggerFactory #:nodoc:
    class << self

      # we want to log iff root.lev <= lev && logger.lev <= lev
      # BTW, root is guaranteed to be defined by this point
      def define_methods(logger)
        return if logger.is_root?

        undefine_methods(logger)

        globlev = Repository['root'].level
        return if logger.level == OFF || globlev == OFF

        toggle_methods(globlev, logger)
      end

      # set logging methods to null defaults
      def undefine_methods(logger)
        for mname in LNAMES
          next if mname == 'OFF'|| mname == 'ALL'
          unset_log(logger, mname)
          set_false(logger, mname)
        end
        set_false(logger, 'all')
        set_true(logger, 'off')
      end
      
      # toggle methods >= globlev that are also >= level
      def toggle_methods(globlev, logger)
        for lev in globlev...LEVELS # satisfies >= globlev
          next if lev < logger.level # satisfies >= level
          next if LNAMES[lev] == 'OFF'
          next if LNAMES[lev] == 'ALL'
          set_log(logger, LNAMES[lev])
          set_true(logger, LNAMES[lev])
        end
        if logger.level == ALL
          set_true(logger, 'all')
        end
        if logger.level != OFF && globlev != OFF
          set_false(logger, 'off')
        end
      end

      # And now, the weird dynamic method definitions! :)
    
      def unset_log(logger, mname)
        mstr="def logger.#{mname.downcase}(data, tracer=nil, logger=self); end"
        module_eval mstr
      end
      
      # performance, sir, performance
      # herein lies the big bottleneck
      def set_log(logger, mname) 
        # create a caller iff the logger invoked is tracing
        tracercall = "nil"
        tracercall = "caller" if logger.trace
        if logger.additive && !logger.parent.is_root?
          # pass parent whatever tracer and logger were created
          # we define the log method such that the trace is only
          # generated by the invoked logger
          parentcall = "@parent.#{mname.downcase}(data, tracer, logger)"
        end
        mstr = %{
          def logger.#{mname.downcase}(data, tracer=#{tracercall}, logger=self)
            for out in @outputters
              out.#{mname.downcase}(logger, tracer, data)
            end
            #{parentcall}
          end
        }
        module_eval mstr
      end
      
      def set_false(logger, mname)
        module_eval "def logger.#{mname.downcase}?; false end"
      end
  
      def set_true(logger, mname)
        module_eval "def logger.#{mname.downcase}?; true end"
      end

    end
  end

end
end
