# This file contains the convenience outputters described in
# log4r/outputter.rb
#
# Version:: $Id: outputters.rb,v 1.3 2002/01/16 12:23:40 cepheus Exp $

require "thread"

require "log4r/outputterfactory"
require "log4r/formatter"

module Log4r
  # Same as Outputter(name, $stdout)
  class StdoutOutputter < Outputter
    def initialize(_name, hash=nil)
      validate_hash(hash)
      @name = _name
      @out = $stdout
      @@outputters[@name] = self
    end
  end

  # Same as Outputter(name, $stderr)
  class StderrOutputter < Outputter
    def initialize(_name, hash=nil)
      validate_hash(hash)
      @name = _name
      @out = $stderr
      @@outputters[@name] = self
    end
  end

  # Extends one of the predefined Outputters, currently StderrOutputter.
  class DefaultOutputter < StderrOutputter #:nodoc:
  end
  
  # Convenience wrapper for File. Additional hash arguments are:
  #
  # [<tt>:filename</tt>]   Name of the file to log to.
  # [<tt>:trunc</tt>]      Truncate the file?
  class FileOutputter < Outputter
    attr_reader :trunc, :filename
    def initialize(_name, hash=nil)
      if _name.nil? or hash.nil?
        raise ArgumentError, "no arguments specified", caller
      end
      validate_hash(hash)
      @name = _name
      
      _filename = (hash[:filename] or hash['filename'] or nil)
      _trunc = hash[:trunc]
      _trunc = hash['trunc'] if _trunc.nil?
      case _trunc
      when false, "false" then @trunc = false
      else @trunc = true
      end

      raise TypeError, "Filename must be specified" if _filename.nil?
      if _filename.type != String
        raise TypeError, "Argument 'filename' must be a String", caller
      end
      @filename = _filename
      @out = File.new(@filename, (@trunc ? "w" : "a")) 
      @@outputters[@name] = self
    end

    class RingBuffOutputter < FileOutputter
      def initialize(name, hash)
      end
    end
  end
end
